#!/usr/bin/env python2.7

import sys
sys.path.insert(0, '..')

from cStringIO import StringIO
from test_msg_pb2 import TestMsg
from random import randint

import log_config

from protobufp.processor import Processor

def get_random_message():
    rand = lambda: randint(11111111, 99999999)

    t = TestMsg()
    t.left = rand()
    t.center = "abc"
    t.right = rand()

    return t

messages = [get_random_message() for i in xrange(5)]

msg_types = [TestMsg]

p = Processor(msg_types)
s = StringIO()

# Build the message stream.

i = 0
for msg in messages:
    print("Pushing message (%d)." % (i))
    s.write(p.serializer.serialize(msg))

    i += 1

# Push the stream into the processor.

p.push(s.getvalue())

# Pop the newest message off the processor, one by one, until they're depleted.

j = 0
while 1:
    in_msg = p.read_message()
    if in_msg is None:
        break

    print("Read message (%d) of type [%s], and now verifying." % 
          (j, in_msg.__class__.__name__))

    assert messages[j].left == in_msg.left
    assert messages[j].center == in_msg.center
    assert messages[j].right == in_msg.right

    j += 1




import logging

from struct import pack, unpack
from cStringIO import StringIO


class Serializer(object):
    def __init__(self, processor):
        self.__log = logging.getLogger(self.__class__.__name__)
        self.__p = processor
        self.__msg_types = self.__p.msg_types

        msg_types_r = {}
        i = 0
        for cls in self.__msg_types:
            msg_types_r[cls.__name__] = i
            i += 1

        self.__msg_types_r = msg_types_r

    def serialize(self, o):
        type_ = self.__msg_types_r[o.__class__.__name__]
        type_string = pack('!I', type_)

        serialized = o.SerializeToString()

        s = StringIO()
        s.write(pack('!I', 4 + len(serialized)))
        s.write(type_string)
        s.write(serialized)

        return s.getvalue()

    def unserialize(self, serialized):
        (type_,) = unpack('!I', serialized[:4])

        o = self.__msg_types[type_]()
        o.ParseFromString(serialized[4:])

        return o




# Generated by the protocol buffer compiler.  DO NOT EDIT!

from google.protobuf import descriptor
from google.protobuf import message
from google.protobuf import reflection
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)



DESCRIPTOR = descriptor.FileDescriptor(
  name='test_msg.proto',
  package='',
  serialized_pb='\n\x0etest_msg.proto\"6\n\x07TestMsg\x12\x0c\n\x04left\x18\x01 \x02(\r\x12\x0e\n\x06\x63\x65nter\x18\x02 \x02(\t\x12\r\n\x05right\x18\x03 \x02(\r')




_TESTMSG = descriptor.Descriptor(
  name='TestMsg',
  full_name='TestMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='left', full_name='TestMsg.left', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='center', full_name='TestMsg.center', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='right', full_name='TestMsg.right', index=2,
      number=3, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=18,
  serialized_end=72,
)

DESCRIPTOR.message_types_by_name['TestMsg'] = _TESTMSG

class TestMsg(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _TESTMSG
  
  # @@protoc_insertion_point(class_scope:TestMsg)

# @@protoc_insertion_point(module_scope)



from protobufp.serializer import Serializer
from protobufp.read_buffer import ReadBuffer


class Processor(object):
    """Automatically identify and unserialize messages.
    
    msg_types is the list of recognized message classes.
    """

    def __init__(self, msg_types):
        self.__msg_types = msg_types
        self.__rb = ReadBuffer()

    def push(self, data):
        self.__rb.push(data)

    def read_message(self):
        data = self.__rb.read_message()
        if data is not None:
            return self.serializer.unserialize(data)

    @property
    def serializer(self):
        try:
            return self.__serializer
        except AttributeError:
            self.__serializer = Serializer(self)
            return self.__serializer

    @property
    def msg_types(self):
        return self.__msg_types




from setuptools import setup, find_packages
import sys, os

setup(name='protobufp',
      version='0.2.3',
      description="",
      long_description="",
      classifiers=[],
      keywords='protobuf translator translation',
      author='Dustin Oprea',
      author_email='myselfasunder@gmail.com',
      url='',
      license='GPL 2',
      packages=find_packages(exclude=['test']),
      include_package_data=True,
      zip_safe=False,
      install_requires=[],
      entry_points="",
)

